@@ -34,13 +34,25 @@ def main():
     parser = optparse.OptionParser(usage=usage, version=version_string)
     parser.allow_interspersed_args = False
 
-    def dash_m_callback(option: str, opt: str, value: str, parser: optparse.OptionParser):
-        parser.values.module_name = value  # type: ignore
+    def store_and_consume_remaining(
+        option: optparse.Option, opt: str, value: str, parser: optparse.OptionParser
+    ):
+        """
+        A callback for optparse that stores the value and consumes all
+        remaining arguments, storing them in the same variable as a tuple.
+        """
 
-        # everything after the -m argument should be passed to that module
-        parser.values.module_args = parser.rargs + parser.largs  # type: ignore
-        parser.rargs[:] = []  # type: ignore
-        parser.largs[:] = []  # type: ignore
+        # assert a few things we know to be true about the parser
+        assert option.dest
+        assert parser.rargs is not None
+        assert parser.largs is not None
+
+        # everything after this argument should be consumed
+        remaining_arguments = parser.rargs + parser.largs
+        parser.rargs[:] = []
+        parser.largs[:] = []
+
+        setattr(parser.values, option.dest, ValueWithRemainingArgs(value, remaining_arguments))
 
     parser.add_option(
         "--load",
@@ -62,12 +74,21 @@ def main():
     parser.add_option(
         "-m",
         "",
-        dest="module_name",
+        dest="module",
         action="callback",
-        callback=dash_m_callback,
-        type="str",
+        callback=store_and_consume_remaining,
+        type="string",
         help="run library module as a script, like 'python -m module'",
     )
+    parser.add_option(
+        "-c",
+        "",
+        dest="program",
+        action="callback",
+        callback=store_and_consume_remaining,
+        type="string",
+        help="program passed in as string, like 'python -c \"...\"'",
+    )
     parser.add_option(
         "",
         "--from-path",
@@ -244,7 +265,8 @@ def main():
     session_options_used = [
         options.load is not None,
         options.load_prev is not None,
-        options.module_name is not None,
+        options.module is not None,
+        options.program is not None,
         len(args) > 0,
     ]
     if session_options_used.count(True) == 0:
@@ -253,16 +275,24 @@ def main():
     if session_options_used.count(True) > 1:
         parser.error("You can only specify one of --load, --load-prev, -m, or script arguments")
 
-    if options.module_name is not None and options.from_path:
+    if options.module is not None and options.from_path:
         parser.error("The options -m and --from-path are mutually exclusive.")
 
     if options.from_path and sys.platform == "win32":
         parser.error("--from-path is not supported on Windows")
 
+    renderer_class = get_renderer_class(options)
+
     # open the output file
 
     if options.outfile:
-        f = open(options.outfile, "w", encoding="utf-8", errors="surrogateescape")
+        f = open(
+            options.outfile,
+            "w",
+            encoding="utf-8",
+            errors="surrogateescape",
+            newline="" if renderer_class.output_is_binary else None,
+        )
         should_close_f_after_writing = True
     else:
         f = sys.stdout
@@ -271,7 +301,7 @@ def main():
     # create the renderer
 
     try:
-        renderer = create_renderer(options, output_file=f)
+        renderer = create_renderer(renderer_class, options, output_file=f)
     except OptionsParseError as e:
         parser.error(e.args[0])
         exit(1)
@@ -289,16 +319,23 @@ def main():
     elif options.load:
         session = Session.load(options.load)
     else:
-        if options.module_name is not None:
+        # we are running some code
+        if options.module is not None:
             if not (sys.path[0] and os.path.samefile(sys.path[0], ".")):
                 # when called with '-m', search the cwd for that module
                 sys.path[0] = os.path.abspath(".")
 
-            sys.argv[:] = [options.module_name] + options.module_args
+            argv = [options.module.value] + options.module.remaining_args
             code = "run_module(modname, run_name='__main__', alter_sys=True)"
-            globs = {"run_module": runpy.run_module, "modname": options.module_name}
+            globs = {"run_module": runpy.run_module, "modname": options.module.value}
+        elif options.program is not None:
+            argv = ["-c", *options.program.remaining_args]
+            code = options.program.value
+            globs = {"__name__": "__main__"}
+            # set the first path entry to '' to match behaviour of python -c
+            sys.path[0] = ""
         else:
-            sys.argv[:] = args
+            argv = args
             if options.from_path:
                 progname = shutil.which(args[0])
                 if progname is None:
@@ -314,18 +351,22 @@ def main():
             code = "run_path(progname, run_name='__main__')"
             globs = {"run_path": runpy.run_path, "progname": progname}
 
+        old_argv = sys.argv.copy()
+
         # there is no point using async mode for command line invocation,
         # because it will always be capturing the whole program, we never want
         # any execution to be <out-of-context>, and it avoids duplicate
         # profiler errors.
         profiler = Profiler(interval=options.interval, async_mode="disabled")
-
         profiler.start()
 
         try:
+            sys.argv[:] = argv
             exec(code, globs, None)
         except (SystemExit, KeyboardInterrupt):
             pass
+        finally:
+            sys.argv[:] = old_argv
 
         session = profiler.stop()
 
@@ -338,7 +379,7 @@ def main():
         if should_close_f_after_writing:
             f.close()
 
-    if options.renderer == "text":
+    if isinstance(renderer, renderers.ConsoleRenderer) and not options.outfile:
         _, report_identifier = save_report_to_temp_storage(session)
         print("To view this report with different options, run:")
         print("    pyinstrument --load-prev %s [options]" % report_identifier)
@@ -421,17 +462,9 @@ class OptionsParseError(Exception):
     pass
 
 
-def create_renderer(options: CommandLineOptions, output_file: TextIO) -> renderers.Renderer:
-    if options.output_html:
-        options.renderer = "html"
-
-    if options.renderer is None and options.outfile:
-        options.renderer = guess_renderer_from_outfile(options.outfile)
-
-    if options.renderer is None:
-        options.renderer = "text"
-
-    renderer_class = get_renderer_class(options.renderer)
+def create_renderer(
+    renderer_class: type[renderers.Renderer], options: CommandLineOptions, output_file: TextIO
+) -> renderers.Renderer:
     render_options = compute_render_options(
         options, renderer_class=renderer_class, output_file=output_file
     )
@@ -445,7 +478,18 @@ def create_renderer(options: CommandLineOptions, output_file: TextIO) -> rendere
         )
 
 
-def get_renderer_class(renderer: str) -> type[renderers.Renderer]:
+def get_renderer_class(options: CommandLineOptions) -> type[renderers.Renderer]:
+    renderer = options.renderer
+
+    if options.output_html:
+        renderer = "html"
+
+    if renderer is None and options.outfile:
+        renderer = guess_renderer_from_outfile(options.outfile)
+
+    if renderer is None:
+        renderer = "text"
+
     if renderer == "text":
         return renderers.ConsoleRenderer
     elif renderer == "html":
@@ -537,8 +581,8 @@ class CommandLineOptions:
     A type that codifies the `options` variable.
     """
 
-    module_name: str | None
-    module_args: list[str]
+    module: ValueWithRemainingArgs | None
+    program: ValueWithRemainingArgs | None
     load: str | None
     load_prev: str | None
     from_path: str | None
@@ -558,5 +602,11 @@ class CommandLineOptions:
     interval: float
 
 
+class ValueWithRemainingArgs:
+    def __init__(self, value: str, remaining_args: list[str]):
+        self.value = value
+        self.remaining_args = remaining_args
+
+
 if __name__ == "__main__":
     main()
